use std::collections::HashMap;

use lazy_static::lazy_static;
use regex_lite::Regex;

use crate::ranged_number_allocator::RangedNumberAllocator;

pub struct HostnamesAllocator {
    allocators_by_type: HashMap<String, RangedNumberAllocator>,
}

lazy_static! {
    static ref HOST_NAME_REGEX: Regex = Regex::new(r"^(\D+)(\d+)$").unwrap();
}

impl HostnamesAllocator {
    pub fn new() -> HostnamesAllocator {
        HostnamesAllocator { allocators_by_type: HashMap::new() }
    }

    fn get_allocator_for_host_type(&mut self, host_type: &str) -> &mut RangedNumberAllocator {
        self.allocators_by_type.entry(host_type.into()).or_insert_with(|| RangedNumberAllocator::new())
    }

    pub fn allocate(&mut self, host_type: &str) -> String {
        let allocator = self.get_allocator_for_host_type(host_type);
        let number = allocator.allocate();
        return format!("{host_type}{number}");
    }

    pub fn free(&mut self, host_name: &str) {
        let (_, [host_type, host_number]) = HOST_NAME_REGEX.captures(host_name).unwrap().extract();
        let allocator = self.get_allocator_for_host_type(host_type);
        allocator.remove(host_number.parse().unwrap());
    }
}
