use std::io::{self, BufRead};

use lazy_static::lazy_static;
use regex_lite::Regex;

use hostnames_allocator::hostnames_allocator::HostnamesAllocator;

lazy_static! {
    static ref COMMAND_REGEX: Regex = Regex::new(r"^([AD])\s+(\w+)$").unwrap();
}

fn main() {
    let mut hostnames_allocator = HostnamesAllocator::new();
    let stdin = io::stdin();
    for line_result in stdin.lock().lines() {
        let line = line_result.unwrap();
        let (_, [command, argument]) = COMMAND_REGEX.captures(&line).unwrap().extract();
        match command {
            "A" => println!("{}", hostnames_allocator.allocate(argument)),
            "D" => hostnames_allocator.free(argument),
            _ => panic!(),
        }
    }
}
